@@ -42,7 +42,8 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <mqtt3.h>
 #include <client_shared.h>
 
-static char *topic = NULL;
+static char **topics = NULL;
+static int topic_count = 0;
 static int topic_qos = 0;
 static mqtt3_context *gcontext;
 int verbose = 0;
@@ -50,11 +51,17 @@ int verbose = 0;
 int my_publish_callback(const char *topic, int qos, uint32_t payloadlen, const uint8_t *payload, int retain)
 {
 	if(verbose){
-		printf("%s %s\n", topic, payload);
+		if(payloadlen){
+			printf("%s %s\n", topic, payload);
+		}else{
+			printf("%s (null)\n", topic);
+		}
 		fflush(stdout);
 	}else{
-		printf("%s\n", payload);
-		fflush(stdout);
+		if(payloadlen){
+			printf("%s\n", payload);
+			fflush(stdout);
+		}
 	}
 
 	return 0;
@@ -62,8 +69,15 @@ int my_publish_callback(const char *topic, int qos, uint32_t payloadlen, const u
 
 void my_connack_callback(int result)
 {
+	int i;
 	if(!result){
-		mqtt3_raw_subscribe(gcontext, false, topic, topic_qos);
+		if(topics){
+			for(i=0; i<topic_count; i++){
+				mqtt3_raw_subscribe(gcontext, false, topics[i], topic_qos);
+			}
+		}else{
+			mqtt3_raw_subscribe(gcontext, false, "#", topic_qos);
+		}
 	}else{
 		fprintf(stderr, "Connect failed\n");
 	}
@@ -72,7 +86,8 @@ void my_connack_callback(int result)
 void print_usage(void)
 {
 	printf("mosquitto_sub is a simple mqtt client that will subscribe to a single topic and print all messages it receives.\n\n");
-	printf("Usage: mosquitto_sub [-h host] [-i id] [-k keepalive] [-p port] [-q qos] [-t topic] [-v]\n\n");
+	printf("Usage: mosquitto_sub [-c] [-i id] [-k keepalive] [-p port] [-q qos] [-t topic] [-v]\n\n");
+	printf(" -c : disable 'clean session' (store subscription and pending messages when client disconnects).\n");
 	printf(" -h : mqtt host to connect to. Defaults to localhost.\n");
 	printf(" -i : id to use for this client. Defaults to mosquitto_sub_ appended with the process id.\n");
 	printf(" -k : keep alive in seconds for this client. Defaults to 60.\n");
@@ -90,9 +105,9 @@ int main(int argc, char *argv[])
 	char *host = "localhost";
 	int port = 1883;
 	int keepalive = 60;
+	bool clean_session = true;
 
 	sprintf(id, "mosquitto_sub_%d", getpid());
-	topic = "#";
 
 	for(i=1; i<argc; i++){
 		if(!strcmp(argv[i], "-p") || !strcmp(argv[i], "--port")){
@@ -109,6 +124,8 @@ int main(int argc, char *argv[])
 				}
 			}
 			i++;
+		}else if(!strcmp(argv[i], "-c") || !strcmp(argv[i], "--disable-clean-session")){
+			clean_session = false;
 		}else if(!strcmp(argv[i], "-h") || !strcmp(argv[i], "--host")){
 			if(i==argc-1){
 				fprintf(stderr, "Error: -h argument given but no host specified.\n\n");
@@ -162,7 +179,9 @@ int main(int argc, char *argv[])
 				print_usage();
 				return 1;
 			}else{
-				topic = argv[i+1];
+				topic_count++;
+				topics = mqtt3_realloc(topics, topic_count*sizeof(char *));
+				topics[topic_count-1] = argv[i+1];
 			}
 			i++;
 		}else if(!strcmp(argv[i], "-v") || !strcmp(argv[i], "--verbose")){
@@ -180,7 +199,7 @@ int main(int argc, char *argv[])
 	client_publish_callback = my_publish_callback;
 	client_connack_callback = my_connack_callback;
 
-	if(client_connect(&context, host, port, id, keepalive)){
+	if(client_connect(&context, host, port, id, keepalive, clean_session)){
 		fprintf(stderr, "Unable to connect.\n");
 		return 1;
 	}
