@@ -104,7 +104,7 @@ void handle_sigint(int signal)
 /* Signal handler for SIGUSR1 - backup the db. */
 void handle_sigusr1(int signal)
 {
-	mqtt3_db_backup();
+	mqtt3_db_backup(false);
 }
 
 int main(int argc, char *argv[])
@@ -122,6 +122,7 @@ int main(int argc, char *argv[])
 	mqtt3_config config;
 	time_t start_time = time(NULL);
 	time_t last_backup = time(NULL);
+	time_t last_store_clean = time(NULL);
 	char buf[1024];
 	int i;
 	FILE *pid;
@@ -164,11 +165,6 @@ int main(int argc, char *argv[])
 		contexts[i] = NULL;
 	}
 
-	signal(SIGINT, handle_sigint);
-	signal(SIGTERM, handle_sigint);
-	signal(SIGUSR1, handle_sigusr1);
-	signal(SIGPIPE, SIG_IGN);
-
 	sigemptyset(&sigblock);
 	sigaddset(&sigblock, SIGINT);
 
@@ -183,9 +179,9 @@ int main(int argc, char *argv[])
 
 	/* Set static $SYS messages */
 	snprintf(buf, 1024, "mosquitto version %s", VERSION);
-	mqtt3_db_messages_queue("$SYS/broker/version", 2, strlen(buf), (uint8_t *)buf, 1);
+	mqtt3_db_messages_easy_queue("", "$SYS/broker/version", 2, strlen(buf), (uint8_t *)buf, 1);
 	snprintf(buf, 1024, "%s", TIMESTAMP);
-	mqtt3_db_messages_queue("$SYS/broker/timestamp", 2, strlen(buf), (uint8_t *)buf, 1);
+	mqtt3_db_messages_easy_queue("", "$SYS/broker/timestamp", 2, strlen(buf), (uint8_t *)buf, 1);
 
 	listensock = mqtt3_malloc(sizeof(int)*config.iface_count);
 	for(i=0; i<config.iface_count; i++){
@@ -204,6 +200,17 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	signal(SIGINT, handle_sigint);
+	signal(SIGTERM, handle_sigint);
+	signal(SIGUSR1, handle_sigusr1);
+	signal(SIGPIPE, SIG_IGN);
+
+	for(i=0; i<config.bridge_count; i++){
+		if(mqtt3_bridge_new(contexts, &context_count, &(config.bridges[i]))){
+			mqtt3_log_printf(MQTT3_LOG_WARNING, "Warning: Unable to connect to bridge %s.", 
+					config.bridges[i].name);
+		}
+	}
 	run = 1;
 	while(run){
 		mqtt3_db_sys_update(config.sys_interval, start_time);
@@ -223,7 +230,10 @@ int main(int argc, char *argv[])
 		for(i=0; i<context_count; i++){
 			if(contexts[i]){
 				if(contexts[i]->sock != -1){
-					if(!(contexts[i]->keepalive) || now - contexts[i]->last_msg_in < contexts[i]->keepalive*3/2){
+					if(contexts[i]->bridge){
+						mqtt3_check_keepalive(contexts[i]);
+					 }
+					 if(!(contexts[i]->keepalive) || now - contexts[i]->last_msg_in < contexts[i]->keepalive*3/2){
 						if(mqtt3_db_message_write(contexts[i])){
 							// FIXME - do something here.
 						}
@@ -240,7 +250,18 @@ int main(int argc, char *argv[])
 						mqtt3_context_cleanup(contexts[i]);
 						contexts[i] = NULL;
 					}
-				}else if(contexts[i]->clean_start){
+				}else if(contexts[i]->bridge){
+					/* Want to try to restart the bridge connection */
+					if(!contexts[i]->bridge->restart_t){
+						contexts[i]->bridge->restart_t = time(NULL)+30;
+					}else{
+						if(time(NULL) > contexts[i]->bridge->restart_t){
+							mqtt3_log_printf(MQTT3_LOG_INFO, "Attempting to reconnect to bridge %s.", contexts[i]->id);
+							contexts[i]->bridge->restart_t = 0;
+							mqtt3_bridge_connect(contexts[i]);
+						}
+					}
+				}else if(contexts[i]->clean_session){
 					mqtt3_context_cleanup(contexts[i]);
 					contexts[i] = NULL;
 				}
@@ -262,9 +283,13 @@ int main(int argc, char *argv[])
 				for(i=0; i<context_count; i++){
 					if(contexts[i] && fstat(contexts[i]->sock, &statbuf) == -1){
 						if(errno == EBADF){
-							mqtt3_log_printf(MQTT3_LOG_NOTICE, "Socket error on client %s, disconnecting.", contexts[i]->id);
+							if(!contexts[i]->disconnecting){
+								mqtt3_log_printf(MQTT3_LOG_NOTICE, "Socket error on client %s, disconnecting.", contexts[i]->id);
+								mqtt3_db_client_will_queue(contexts[i]);
+							}else{
+								mqtt3_log_printf(MQTT3_LOG_NOTICE, "Client %s disconnected.", contexts[i]->id);
+							}
 							contexts[i]->sock = -1;
-							if(!contexts[i]->disconnecting) mqtt3_db_client_will_queue(contexts[i]);
 							mqtt3_context_cleanup(contexts[i]);
 							contexts[i] = NULL;
 						}
@@ -275,20 +300,38 @@ int main(int argc, char *argv[])
 			for(i=0; i<context_count; i++){
 				if(contexts[i] && contexts[i]->sock != -1 && FD_ISSET(contexts[i]->sock, &writefds)){
 					if(mqtt3_net_write(contexts[i])){
-						mqtt3_log_printf(MQTT3_LOG_NOTICE, "Socket write error on client %s, disconnecting.", contexts[i]->id);
+						if(!contexts[i]->disconnecting){
+							mqtt3_log_printf(MQTT3_LOG_NOTICE, "Socket write error on client %s, disconnecting.", contexts[i]->id);
+							mqtt3_db_client_will_queue(contexts[i]);
+						}else{
+							mqtt3_log_printf(MQTT3_LOG_NOTICE, "Client %s disconnected.", contexts[i]->id);
+						}
 						/* Write error or other that means we should disconnect */
-						if(!contexts[i]->disconnecting) mqtt3_db_client_will_queue(contexts[i]);
-						mqtt3_context_cleanup(contexts[i]);
-						contexts[i] = NULL;
+						/* Bridges don't get cleaned up because they will reconnect later. */
+						if(contexts[i]->bridge){
+							mqtt3_socket_close(contexts[i]);
+						}else{
+							mqtt3_context_cleanup(contexts[i]);
+							contexts[i] = NULL;
+						}
 					}
 				}
 				if(contexts[i] && contexts[i]->sock != -1 && FD_ISSET(contexts[i]->sock, &readfds)){
 					if(mqtt3_net_read(contexts[i])){
-						mqtt3_log_printf(MQTT3_LOG_NOTICE, "Socket read error on client %s, disconnecting.", contexts[i]->id);
+						if(!contexts[i]->disconnecting){
+							mqtt3_log_printf(MQTT3_LOG_NOTICE, "Socket read error on client %s, disconnecting.", contexts[i]->id);
+							if(!contexts[i]->disconnecting) mqtt3_db_client_will_queue(contexts[i]);
+						}else{
+							mqtt3_log_printf(MQTT3_LOG_NOTICE, "Client %s disconnected.", contexts[i]->id);
+						}
 						/* Read error or other that means we should disconnect */
-						if(!contexts[i]->disconnecting) mqtt3_db_client_will_queue(contexts[i]);
-						mqtt3_context_cleanup(contexts[i]);
-						contexts[i] = NULL;
+						/* Bridges don't get cleaned up because they will reconnect later. */
+						if(contexts[i]->bridge){
+							mqtt3_socket_close(contexts[i]);
+						}else{
+							mqtt3_context_cleanup(contexts[i]);
+							contexts[i] = NULL;
+						}
 					}
 				}
 			}
@@ -300,10 +343,14 @@ int main(int argc, char *argv[])
 		}
 		if(config.persistence && config.autosave_interval){
 			if(last_backup + config.autosave_interval < now){
-				mqtt3_db_backup();
+				mqtt3_db_backup(false);
 				last_backup = time(NULL);
 			}
 		}
+		if(!config.store_clean_interval || last_store_clean + config.store_clean_interval < now){
+			mqtt3_db_store_clean();
+			last_store_clean = time(NULL);
+		}
 	}
 
 	mqtt3_log_printf(MQTT3_LOG_INFO, "mosquitto version %s terminating", VERSION);
@@ -327,7 +374,7 @@ int main(int argc, char *argv[])
 	}
 
 	if(config.persistence && config.autosave_interval){
-		mqtt3_db_backup();
+		mqtt3_db_backup(true);
 	}
 	mqtt3_db_close();
 
