@@ -48,7 +48,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #define PROTOCOL_VERSION 3
 
 /* Database macros */
-#define MQTT_DB_VERSION 1
+#define MQTT_DB_VERSION 2
 
 /* Macros for accessing the MSB and LSB of a uint16_t */
 #define MQTT_MSB(A) (uint8_t)((A & 0xFF00) >> 8)
@@ -101,18 +101,39 @@ struct _mqtt3_packet{
 	struct _mqtt3_packet *next;
 };
 
+typedef enum {
+	bd_out = 0,
+	bd_in = 1,
+	bd_both = 2
+} mqtt3_bridge_direction;
+
+struct _mqtt3_bridge_topic{
+	char *topic;
+	mqtt3_bridge_direction direction;
+};
+
+struct _mqtt3_bridge{
+	char *name;
+	char *address;
+	uint16_t port;
+	struct _mqtt3_bridge_topic *topics;
+	int topic_count;
+	time_t restart_t;
+};
+
 typedef struct _mqtt3_context{
 	int sock;
 	time_t last_msg_in;
 	time_t last_msg_out;
 	uint16_t keepalive;
-	bool clean_start;
+	bool clean_session;
 	bool connected;
 	bool disconnecting;
 	char *id;
 	char *address;
 	struct _mqtt3_packet in_packet;
 	struct _mqtt3_packet *out_packet;
+	struct _mqtt3_bridge *bridge;
 } mqtt3_context;
 
 typedef enum {
@@ -148,12 +169,25 @@ typedef struct {
 	int log_type;
 	bool persistence;
 	char *persistence_location;
+	char *persistence_file;
 	int retry_interval;
+	int store_clean_interval;
 	int sys_interval;
 	char *pid_file;
 	char *user;
+	struct _mqtt3_bridge *bridges;
+	int bridge_count;
 } mqtt3_config;
 
+struct _mqtt3_listener {
+	int fd;
+	char *iface;
+	uint16_t port;
+	int max_connections;
+	char *mount_point;
+	char *client_prefix;
+};
+
 #ifdef WITH_CLIENT
 /* Client callback for publish events - this WILL change. */
 extern int (*client_publish_callback)(const char *, int, uint32_t, const uint8_t *, int);
@@ -172,6 +206,7 @@ extern void (*client_net_write_callback)(int);
  * ============================================================ */
 /* Return a string that corresponds to the MQTT command number (left shifted 4 bits). */
 const char *mqtt3_command_to_string(uint8_t command);
+void mqtt3_check_keepalive(mqtt3_context *context);
 /* Remove excessive slashes in a subscription/topic. */
 int mqtt3_fix_sub_topic(char **subtopic);
 
@@ -195,7 +230,7 @@ int mqtt3_config_read(mqtt3_config *config, const char *filename);
  */
 int mqtt3_send_command_with_mid(mqtt3_context *context, uint8_t command, uint16_t mid);
 int mqtt3_raw_connack(mqtt3_context *context, uint8_t result);
-int mqtt3_raw_connect(mqtt3_context *context, const char *client_id, bool will, uint8_t will_qos, bool will_retain, const char *will_topic, const char *will_msg, uint16_t keepalive, bool clean_start);
+int mqtt3_raw_connect(mqtt3_context *context, const char *client_id, bool will, uint8_t will_qos, bool will_retain, const char *will_topic, const char *will_msg, uint16_t keepalive, bool clean_session);
 int mqtt3_raw_disconnect(mqtt3_context *context);
 int mqtt3_raw_pingreq(mqtt3_context *context);
 int mqtt3_raw_pingresp(mqtt3_context *context);
@@ -260,7 +295,7 @@ int mqtt3_handle_unsubscribe(mqtt3_context *context);
  * ============================================================ */
 int mqtt3_db_open(mqtt3_config *config);
 int mqtt3_db_close(void);
-int mqtt3_db_backup(void);
+int mqtt3_db_backup(bool cleanup);
 int mqtt3_db_client_insert(mqtt3_context *context, int will, int will_retain, int will_qos, const char *will_topic, const char *will_message);
 int mqtt3_db_client_update(mqtt3_context *context, int will, int will_retain, int will_qos, const char *will_topic, const char *will_message);
 int mqtt3_db_client_count(int *count);
@@ -272,33 +307,36 @@ int mqtt3_db_client_find_socket(const char *client_id, int *sock);
 int mqtt3_db_client_invalidate_socket(const char *client_id, int sock);
 /* Add the will of the client in context to the queue of clients subscribed to the appropriate topic. */
 int mqtt3_db_client_will_queue(mqtt3_context *context);
+void mqtt3_db_limits_set(int inflight, int queued);
 /* Return the number of in-flight messages in count. */
 int mqtt3_db_message_count(int *count);
 int mqtt3_db_message_delete(const char *client_id, uint16_t mid, mqtt3_msg_direction dir);
-int mqtt3_db_message_delete_by_oid(uint64_t oid);
-int mqtt3_db_message_insert(const char *client_id, uint16_t mid, mqtt3_msg_direction dir, mqtt3_msg_status status, int retain, const char *topic, int qos, uint32_t payloadlen, const uint8_t *payload);
+int mqtt3_db_message_delete_by_oid(int64_t oid);
+int mqtt3_db_message_insert(const char *client_id, uint16_t mid, mqtt3_msg_direction dir, mqtt3_msg_status status, int qos, int64_t store_id);
 int mqtt3_db_message_release(const char *client_id, uint16_t mid, mqtt3_msg_direction dir);
 int mqtt3_db_message_update(const char *client_id, uint16_t mid, mqtt3_msg_direction dir, mqtt3_msg_status status);
 int mqtt3_db_message_write(mqtt3_context *context);
 int mqtt3_db_messages_delete(const char *client_id);
-int mqtt3_db_messages_queue(const char *topic, int qos, uint32_t payloadlen, const uint8_t *payload, int retain);
+int mqtt3_db_messages_easy_queue(const char *client_id, const char *topic, int qos, uint32_t payloadlen, const uint8_t *payload, int retain);
+int mqtt3_db_messages_queue(const char *source_id, const char *topic, int qos, int retain, int64_t store_id);
+int mqtt3_db_message_store(const char *source, const char *topic, int qos, uint32_t payloadlen, const uint8_t *payload, int retain, int64_t *store_id);
 /* Check all messages waiting on a client reply and resend if timeout has been exceeded. */
 int mqtt3_db_message_timeout_check(unsigned int timeout);
 /* Generate an outgoing mid for client_id. */
 uint16_t mqtt3_db_mid_generate(const char *client_id);
-/* Find retained messages for topic and return values in qos, payloadlen and payload. */
-int mqtt3_db_retain_find(const char *topic, int *qos, uint32_t *payloadlen, uint8_t **payload);
 /* Add a retained message for a topic, overwriting an existing one if necessary. */
-int mqtt3_db_retain_insert(const char *topic, int qos, uint32_t payloadlen, const uint8_t *payload);
+int mqtt3_db_retain_insert(const char *topic, int64_t store_id);
+int mqtt3_db_retain_delete(const char *topic);
 int mqtt3_db_retain_queue(mqtt3_context *context, const char *sub, int sub_qos);
+int mqtt3_db_store_clean(void);
 /* Insert a new subscription/qos for a client. */
 int mqtt3_db_sub_insert(const char *client_id, const char *sub, int qos);
 /* Remove a subscription for a client. */
 int mqtt3_db_sub_delete(const char *client_id, const char *sub);
-int mqtt3_db_sub_search_start(const char *topic);
+int mqtt3_db_sub_search_start(const char *source_id, const char *topic);
 int mqtt3_db_sub_search_next(char **client_id, uint8_t *qos);
 /* Remove all subscriptions for a client. */
-int mqtt3_db_subs_clean_start(const char *client_id);
+int mqtt3_db_subs_clean_session(const char *client_id);
 void mqtt3_db_sys_update(int interval, time_t start_time);
 
 /* ============================================================
@@ -325,4 +363,11 @@ int mqtt3_log_init(int level, int destinations);
 int mqtt3_log_close(void);
 int mqtt3_log_printf(int level, const char *fmt, ...) __attribute__((format(printf, 2, 3)));
 
+/* ============================================================
+ * Bridge functions
+ * ============================================================ */
+int mqtt3_bridge_new(mqtt3_context **contexts, int *context_count, struct _mqtt3_bridge *bridge);
+int mqtt3_bridge_connect(mqtt3_context *context);
+void mqtt3_bridge_packet_cleanup(mqtt3_context *context);
+
 #endif
