@@ -56,9 +56,9 @@ static unsigned long msgs_sent = 0;
 void (*client_net_write_callback)(int) = NULL;
 #endif
 
-int _mqtt3_socket_listen(struct sockaddr *addr);
+static int _mqtt3_socket_listen(struct sockaddr *addr);
 
-int mqtt3_socket_accept(mqtt3_context **contexts, int context_count, int listensock)
+int mqtt3_socket_accept(mqtt3_context **contexts, int *context_count, int listensock)
 {
 	int i;
 	int new_sock = -1;
@@ -90,19 +90,20 @@ int mqtt3_socket_accept(mqtt3_context **contexts, int context_count, int listens
 	}else{
 #endif
 		new_context = mqtt3_context_init(new_sock);
+		if(!new_context) return -1;
 		mqtt3_log_printf(MQTT3_LOG_NOTICE, "New client connected from %s.", new_context->address);
-		for(i=0; i<context_count; i++){
+		for(i=0; i<(*context_count); i++){
 			if(contexts[i] == NULL){
 				contexts[i] = new_context;
 				break;
 			}
 		}
-		if(i==context_count){
-			context_count++;
-			tmp_contexts = mqtt3_realloc(contexts, sizeof(mqtt3_context*)*context_count);
+		if(i==(*context_count)){
+			(*context_count)++;
+			tmp_contexts = mqtt3_realloc(contexts, sizeof(mqtt3_context*)*(*context_count));
 			if(tmp_contexts){
 				contexts = tmp_contexts;
-				contexts[context_count-1] = new_context;
+				contexts[(*context_count)-1] = new_context;
 			}
 		}
 #ifdef WITH_WRAP
@@ -182,7 +183,7 @@ int mqtt3_socket_connect(const char *host, uint16_t port)
  * Returns -1 on failure (addr is NULL, socket creation/listening error)
  * Returns sock number on success.
  */
-int _mqtt3_socket_listen(struct sockaddr *addr)
+static int _mqtt3_socket_listen(struct sockaddr *addr)
 {
 	int sock;
 	int opt = 1;
@@ -313,7 +314,7 @@ int mqtt3_net_read(mqtt3_context *context)
 			context->in_packet.command = byte;
 #ifdef WITH_BROKER
 			/* Clients must send CONNECT as their first command. */
-			if(context->connected == false && (byte&0xF0) != CONNECT) return 1;
+			if(!(context->bridge) && context->connected == false && (byte&0xF0) != CONNECT) return 1;
 #endif
 		}else{
 			if(read_length == 0) return 1; /* EOF */
