@@ -76,7 +76,7 @@ int mqtt3_packet_handle(mqtt3_context *context)
 		case UNSUBSCRIBE:
 			return mqtt3_handle_unsubscribe(context);
 #endif
-#ifdef WITH_CLIENT
+#if defined(WITH_CLIENT) || defined(WITH_BRIDGE)
 		case CONNACK:
 			return mqtt3_handle_connack(context);
 		case SUBACK:
@@ -143,12 +143,13 @@ int mqtt3_handle_pubcomp(mqtt3_context *context)
 int mqtt3_handle_publish(mqtt3_context *context)
 {
 	char *topic;
-	uint8_t *payload;
+	uint8_t *payload = NULL;
 	uint32_t payloadlen;
 	uint8_t dup, qos, retain;
 	uint16_t mid;
 	int rc = 0;
 	uint8_t header = context->in_packet.command;
+	int64_t store_id = 0;
 
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received PUBLISH from %s", context->id);
 	dup = (header & 0x08)>>3;
@@ -169,30 +170,40 @@ int mqtt3_handle_publish(mqtt3_context *context)
 	}
 
 	payloadlen = context->in_packet.remaining_length - context->in_packet.pos;
-	payload = mqtt3_calloc(payloadlen+1, sizeof(uint8_t));
-	if(mqtt3_read_bytes(context, payload, payloadlen)){
+	if(payloadlen){
+		payload = mqtt3_calloc(payloadlen+1, sizeof(uint8_t));
+		if(mqtt3_read_bytes(context, payload, payloadlen)){
+			mqtt3_free(topic);
+			return 1;
+		}
+	}else{
+		if(retain){
+			/* If retain is set and we have a zero payloadlen, delete the
+			 * retained message because the last message was a null! */
+			rc = mqtt3_db_retain_delete(topic);
+		}
+	}
+
+	if(mqtt3_db_message_store(context->id, topic, qos, payloadlen, payload, retain, &store_id)){
 		mqtt3_free(topic);
+		if(payload) mqtt3_free(payload);
 		return 1;
 	}
-#ifdef DEBUG
-	printf("%s: %s\n", topic, payload);
-#endif
-
 	switch(qos){
 		case 0:
-			if(mqtt3_db_messages_queue(topic, qos, payloadlen, payload, retain)) rc = 1;
+			if(mqtt3_db_messages_queue(context->id, topic, qos, retain, store_id)) rc = 1;
 			break;
 		case 1:
-			if(mqtt3_db_messages_queue(topic, qos, payloadlen, payload, retain)) rc = 1;
+			if(mqtt3_db_messages_queue(context->id, topic, qos, retain, store_id)) rc = 1;
 			if(mqtt3_raw_puback(context, mid)) rc = 1;
 			break;
 		case 2:
-			if(mqtt3_db_message_insert(context->id, mid, md_in, ms_wait_pubrec, retain, topic, qos, payloadlen, payload)) rc = 1;
+			if(mqtt3_db_message_insert(context->id, mid, md_in, ms_wait_pubrec, qos, store_id)) rc = 1;
 			if(mqtt3_raw_pubrec(context, mid)) rc = 1;
 			break;
 	}
 	mqtt3_free(topic);
-	mqtt3_free(payload);
+	if(payload) mqtt3_free(payload);
 
 	return rc;
 }
