@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009, Roger Light <roger@atchoo.org>
+Copyright (c) 2009,2010, Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -40,16 +40,31 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <config.h>
 #include <mqtt3.h>
 
+void (*client_connack_callback)(int) = NULL;
+
 int mqtt3_handle_connack(mqtt3_context *context)
 {
 	uint8_t byte;
 	uint8_t rc;
+	int i;
 
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received CONNACK");
 	if(mqtt3_read_byte(context, &byte)) return 1; // Reserved byte, not used
 	if(mqtt3_read_byte(context, &rc)) return 1;
+	if(client_connack_callback){
+		client_connack_callback(rc);
+	}
 	switch(rc){
 		case 0:
+			if(context->bridge){
+				for(i=0; i<context->bridge->topic_count; i++){
+					if(context->bridge->topics[i].direction == bd_in || context->bridge->topics[i].direction == bd_both){
+						if(mqtt3_raw_subscribe(context, false, context->bridge->topics[i].topic, 2)){
+							return 1;
+						}
+					}
+				}
+			}
 			return 0;
 		case 1:
 			mqtt3_log_printf(MQTT3_LOG_ERR, "Connection Refused: unacceptable protocol version");
