@@ -47,7 +47,7 @@ int mqtt3_handle_connect(mqtt3_context *context)
 	uint8_t connect_flags;
 	char *client_id;
 	char *will_topic = NULL, *will_message = NULL;
-	uint8_t will, will_retain, will_qos, clean_start;
+	uint8_t will, will_retain, will_qos, clean_session;
 	
 	/* Don't accept multiple CONNECT commands. */
 	if(context->connected) return 1;
@@ -77,7 +77,7 @@ int mqtt3_handle_connect(mqtt3_context *context)
 	mqtt3_free(protocol_name);
 
 	if(mqtt3_read_byte(context, &connect_flags)) return 1;
-	clean_start = connect_flags & 0x02;
+	clean_session = connect_flags & 0x02;
 	will = connect_flags & 0x04;
 	will_qos = (connect_flags & 0x18) >> 2;
 	will_retain = connect_flags & 0x20;
@@ -92,7 +92,7 @@ int mqtt3_handle_connect(mqtt3_context *context)
 
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received CONNECT from %s as %s", context->address, client_id);
 	context->id = client_id;
-	context->clean_start = clean_start;
+	context->clean_session = clean_session;
 
 	mqtt3_db_client_insert(context, will, will_retain, will_qos, will_topic, will_message);
 
@@ -120,13 +120,6 @@ int mqtt3_handle_subscribe(mqtt3_context *context)
 	uint8_t *payload = NULL;
 	uint32_t payloadlen = 0;
 
-#ifndef WITH_REGEX
-	uint16_t retain_mid;
-	int retain_qos;
-	uint8_t *retain_payload = NULL;
-	uint32_t retain_payloadlen;
-#endif
-
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received SUBSCRIBE from %s", context->id);
 	/* FIXME - plenty of potential for memory leaks here */
 	if(!context) return 1;
@@ -137,49 +130,32 @@ int mqtt3_handle_subscribe(mqtt3_context *context)
 		sub = NULL;
 		if(mqtt3_read_string(context, &sub)){
 			if(sub) mqtt3_free(sub);
+			if(payload) mqtt3_free(payload);
 			return 1;
 		}
 
-		if(mqtt3_read_byte(context, &qos)) return 1;
 		if(sub){
-			if(mqtt3_fix_sub_topic(&sub)) return 1;
+			if(mqtt3_read_byte(context, &qos)){
+				mqtt3_free(sub);
+				if(payload) mqtt3_free(payload);
+				return 1;
+			}
+			if(mqtt3_fix_sub_topic(&sub)){
+				mqtt3_free(sub);
+				if(payload) mqtt3_free(payload);
+				return 1;
+			}
 			if(!strlen(sub)){
 				mqtt3_log_printf(MQTT3_LOG_INFO, "Empty subscription string from %s, disconnecting.",
 					context->address);
+				mqtt3_free(sub);
+				if(payload) mqtt3_free(payload);
 				return 1;
 			}
 			mqtt3_log_printf(MQTT3_LOG_DEBUG, "\t%s (QoS %d)", sub, qos);
 			mqtt3_db_sub_insert(context->id, sub, qos);
 	
-#ifdef WITH_REGEX
 			if(mqtt3_db_retain_queue(context, sub, qos)) rc = 1;
-#else
-			if(!mqtt3_db_retain_find(sub, &retain_qos, &retain_payloadlen, &retain_payload)){
-				if(retain_qos > qos) retain_qos = qos;
-				if(retain_qos > 0){
-					retain_mid = mqtt3_db_mid_generate(context->id);
-				}else{
-					retain_mid = 0;
-				}
-				switch(retain_qos){
-					case 0:
-						if(mqtt3_db_message_insert(context->id, retain_mid, md_out, ms_publish, 1,
-								sub, retain_qos, retain_payloadlen, retain_payload)) rc = 1;
-						break;
-					case 1:
-						if(mqtt3_db_message_insert(context->id, retain_mid, md_out, ms_publish_puback, 1,
-								sub, retain_qos, retain_payloadlen, retain_payload)) rc = 1;
-						break;
-					case 2:
-						if(mqtt3_db_message_insert(context->id, retain_mid, md_out, ms_publish_pubrec, 1,
-								sub, retain_qos, retain_payloadlen, retain_payload)) rc = 1;
-						break;
-				}
-				if(retain_payload){
-					mqtt3_free(retain_payload);
-				}
-			}
-#endif
 			mqtt3_free(sub);
 		}
 
