@@ -69,6 +69,13 @@ const char *mqtt3_command_to_string(uint8_t command)
 	return "UNKNOWN";
 }
 
+void mqtt3_check_keepalive(mqtt3_context *context)
+{
+	if(time(NULL) - context->last_msg_out >= context->keepalive){
+		mqtt3_raw_pingreq(context);
+	}
+}
+
 /* Convert ////some////over/slashed///topic/etc/etc//
  * into some/over/slashed/topic/etc/etc
  */
@@ -79,7 +86,9 @@ int mqtt3_fix_sub_topic(char **subtopic)
 
 	if(!subtopic || !(*subtopic)) return 1;
 
-	fixed = mqtt3_calloc(strlen(*subtopic)+1, 1);
+	/* size of fixed here is +1 for the terminating 0 and +1 for the spurious /
+	 * that gets appended. */
+	fixed = mqtt3_calloc(strlen(*subtopic)+2, 1);
 	if(!fixed) return 1;
 
 	token = strtok(*subtopic, "/");
